@@ -3,7 +3,8 @@ scam.check <- function(b,rl.col=3,...)
 # takes a fitted scam object and produces some standard diagnostic plots
 {   old.par<-par(mfrow=c(2,2))
     sc.name<-b$method
-    qqnorm(residuals(b),pch=".",...)
+   # qqnorm(residuals(b),pch=".",...)
+    qqnorm(residuals(b),...)
     qqline(residuals(b),col=rl.col,...)
     plot(b$linear.predictors,residuals(b),main="Resids vs. linear pred.",
          xlab="linear predictor",ylab="residuals",...);
